@@ -1,28 +1,47 @@
 # -*- coding: utf-8 -*-
-from django.db import models, transaction
-from django.conf import settings
-from django.core.exceptions import ImproperlyConfigured
-from django.utils.importlib import import_module
-from taggit.managers import TaggableManager
 from datetime import datetime
 import logging
 
+from django.conf import settings
+from django.core.exceptions import ImproperlyConfigured
+from django.db import models, transaction
+from django.utils.importlib import import_module
+from taggit.managers import TaggableManager
+from tyoi.oauth2 import AccessTokenResponseError
+
+from .exceptions import AccountLocked, LoginPasswordError
+
+
 log = logging.getLogger('oauth_tokens')
 
 HISTORY = getattr(settings, 'OAUTH_TOKENS_HISTORY', False)
-PROVIDERS = getattr(settings, 'OAUTH_TOKENS_PROVIDERS', {
-    'vkontakte': 'oauth_tokens.providers.vkontakte.VkontakteAccessToken',
-    'facebook': 'oauth_tokens.providers.facebook.FacebookAccessToken',
-})
-PROVIDER_CHOICES = [((provider, provider.title())) for provider in PROVIDERS.keys()]
+PROVIDERS = [
+    'vkontakte',
+    'facebook',
+    'twitter',
+    'odnoklassniki',
+]
+PROVIDER_CHOICES = [((provider, provider.title())) for provider in PROVIDERS]
+ACCESS_TOKENS_CLASSES = getattr(settings, 'OAUTH_TOKENS_CLASSES',
+                                dict([(p, 'oauth_tokens.providers.%s.%sAccessToken' % (p, p.title()))
+                                      for p in PROVIDERS])
+                                )
+
 
 class AccessTokenGettingError(Exception):
     pass
 
+
+class AccessTokenRefreshingError(Exception):
+    pass
+
+
 class AccessTokenManager(models.Manager):
+
     '''
     Defautl manager for AccessToken for retrieving token
     '''
+
     def filter(self, *args, **kwargs):
         '''
         Optional filter by user's `tag`
@@ -36,24 +55,80 @@ class AccessTokenManager(models.Manager):
     def filter_active_tokens_of_provider(self, provider, *args, **kwargs):
         return self.filter(provider=provider, expires__gt=datetime.now(), *args, **kwargs).order_by('?')
 
+    def get_token(self, provider, tag=None):
+        '''
+        Returns access token instance. If tag argument provided or
+        settings OAUTH_TOKENS_%s_USERNAME is not defined look up for credentials in DB
+        '''
+        token_class = self.get_token_class(provider)
+
+        if tag is None and getattr(settings, 'OAUTH_TOKENS_%s_USERNAME' % provider.upper(), None):
+            return token_class()
+
+        qs_users = UserCredentials.objects.filter(provider=provider)
+        if tag:
+            qs_users = qs_users.filter(tags__name__in=[tag])
+
+        try:
+            user = qs_users[0]
+        except IndexError:
+            raise Exception("User with tag %s for provider %s does not exist" % (tag, provider))
+
+        return self.get_token_by_user(token_class, user)
+
+    def get_token_by_user(self, token_class, user):
+        return token_class(username=user.username, password=user.password, additional=user.additional)
+
+    def get_token_class(self, provider):
+
+        if provider not in PROVIDERS:
+            raise ValueError("Provider `%s` not in available providers list" % provider)
+
+        try:
+            path = ACCESS_TOKENS_CLASSES[provider].split('.')
+            module = '.'.join(path[:-1])
+            class_name = path[-1]
+            token_class = getattr(import_module(module), path[-1])
+        except ImportError:
+            raise ImproperlyConfigured("Impossible to find access token class with path %s" %
+                                       ACCESS_TOKENS_CLASSES[provider])
+
+        return token_class
+
+    def refresh(self, provider):
+        '''
+        Refresh tokens and save as new save it to database
+        '''
+        tokens = AccessToken.objects.filter(provider=provider).order_by('-id')
+
+        access_tokens = []
+        # TODO: remove limit for queryset, but handle behaviour with old accesstokens
+        for token in tokens[:1]:
+            token_class = self.get_token_class(provider)
+
+            try:
+                new_token = token_class().refresh(token)
+            except AccessTokenResponseError:
+                return self.fetch(provider)
+
+            access_token = self.model(provider=provider, user=token.user)
+            access_token.__dict__.update(new_token.__dict__)
+            access_token.refresh_token = token.refresh_token
+            access_token.save()
+            access_tokens += [access_token]
+
+        if len(access_tokens) == 0:
+            raise AccessTokenRefreshingError("No tokens refreshed for provider %s" % provider)
+
+        return access_tokens
+
     @transaction.commit_on_success
     def fetch(self, provider):
         '''
         Get new token and save it to database for all users in UserCredentials table.
         Сlean database before if OAUTH_TOKENS_HISTORY disabled
         '''
-        from base import OAuthError
-
-        if provider not in PROVIDERS:
-            raise ValueError("Provider `%s` not in available providers list" % provider)
-
-        try:
-            path = PROVIDERS[provider].split('.')
-            module = '.'.join(path[:-1])
-            class_name = path[-1]
-            token_class = getattr(import_module(module), path[-1])
-        except ImportError:
-            raise ImproperlyConfigured("Impossible to find access token class with path %s" % PROVIDERS[provider])
+        token_class = self.get_token_class(provider)
 
         # walk through all users of current provider in UserCredentials table
         # or try to get user credentials from settings
@@ -66,11 +141,15 @@ class AccessTokenManager(models.Manager):
         for user in users:
 
             try:
-                token = token_class(user=user).get()
-                assert token
-            except (OAuthError, ImproperlyConfigured), e:
+                token = self.get_token_by_user(token_class, user).get()
+            except AccountLocked, e:
                 log.error(u"Error '%s' while getting new token for provider %s and user %s" % (e, provider, user))
                 continue
+            except LoginPasswordError, e:
+                log.error(u"Error '%s' while getting new token for provider %s and user %s" % (e, provider, user))
+                user.active = False
+                user.save()
+                continue
 
             if not HISTORY:
                 self.filter(provider=provider, user=user).delete()
@@ -81,11 +160,13 @@ class AccessTokenManager(models.Manager):
             access_tokens += [access_token]
 
         if len(access_tokens) == 0:
-            raise AccessTokenGettingError("Error while updating tokens for provider %s" % provider)
+            raise AccessTokenGettingError("No tokens for provider %s" % provider)
 
         return access_tokens
 
+
 class AccessToken(models.Model):
+
     class Meta:
         verbose_name = 'Oauth access token'
         verbose_name_plural = 'Oauth access tokens'
@@ -108,6 +189,7 @@ class AccessToken(models.Model):
     def __str__(self):
         return '#%s' % self.access_token
 
+
 class UserCredentials(models.Model):
 
     name = models.CharField(max_length=100)
@@ -121,4 +203,4 @@ class UserCredentials(models.Model):
     tags = TaggableManager(blank=True)
 
     def __unicode__(self):
-        return self.name
\ No newline at end of file
+        return self.name
